/********************************************************************
 *
 * Motif-Programm zur Password-Eingabe
 *
 * Quelle: BREDEX GmbH, Braunschweig
 *
 ********************************************************************/

/* Allgemeine Header-Dateien */
#include <stdio.h>
#include <stdlib.h>
#include <Xm/Xm.h>

/* Header-Dateien fuer verwendete Widgetklassen */
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/SelectioB.h>
#include <Xm/Text.h>

/* Klassenname der Applikation */
#define APPL_KLASSE    "XmBsp"

/* Vorwaerts-Deklarationen */
void initWidgetBaum();
void getpwCB(); 
void getpwEndeCB(); 
void exitCB(); 


main (argc, argv)
int   argc;
char* argv[];
{
    XtAppContext kontext;        /* Applikations-Umgebung */
    Widget       applShell;      /* Applikations-Shell */

    /* Initialisierungen und Applikations-Shell erzeugen */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 (XrmOptionDescRec*)NULL, 0,
                                 &argc, argv, (String*)NULL,
                                 (Arg*)NULL, 0);

    /* Widgetbaum unter Applikations-Shell aufbauen */
    initWidgetBaum (applShell);

    /* Widgetbaum realisieren */
    XtRealizeWidget (applShell);

    /* Hauptschleife mit Ereignisbehandlung */
    XtAppMainLoop (kontext);
}


void initWidgetBaum (elternWidget)
Widget elternWidget;
{
    Widget box, button;

    /* Button-Container erzeugen */
    box = XtCreateManagedWidget ("box", xmRowColumnWidgetClass,
                                 elternWidget, (Arg*)NULL, 0);
    
    /* XmPush-Button zur Password-Abfrage erzeugen */
    button = XtCreateManagedWidget ("getPassword", xmPushButtonWidgetClass,
                                    box, (Arg*)NULL, 0);
    XtAddCallback (button, XmNactivateCallback, getpwCB, (XtPointer)NULL);

    /* Exit-Button erzeugen */
    button = XtCreateManagedWidget ("exit", xmPushButtonWidgetClass,
                                    box, (Arg*)NULL, 0);
    XtAddCallback (button, XmNactivateCallback, exitCB, (XtPointer)NULL);

}

/* getpwCB
 *  - Dialoganfrage zum Password-Einlesen starten
 */
/*ARGSUSED*/
void getpwCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (nicht verwendet) */
XtPointer aufrufDaten; /* Aufruf-Daten (nicht verwendet) */
{
    static Widget dialogW = NULL;
    Pixel bg;
    int status;
    Arg args[10];
    Cardinal n;

    /* Dialog beim ersten Mal erzeugen
     */
    if (dialogW == NULL) {
        /* Dialoganfrage erzeugen
         *  - Rest des Bildschirms sperren
         *  - bei Eingabeende getpwEndeCB() aufrufen
         */
        n = 0;
        XtSetArg (args[n], XmNdialogStyle, XmDIALOG_SYSTEM_MODAL);  n++;
        dialogW = XmCreatePromptDialog (widget, "pwDialog", args, n);
        XtAddCallback (dialogW, XmNokCallback,
                        getpwEndeCB, (XtPointer)NULL);
        XtAddCallback (dialogW, XmNcancelCallback,
                        getpwEndeCB, (XtPointer)NULL);

        /* der Hilfe-Button wird nicht gebraucht */
        XtDestroyWidget (XmSelectionBoxGetChild(dialogW, XmDIALOG_HELP_BUTTON));

        /* Hintergrundfarbe im Text-Widget abfragen */
        n = 0;
        XtSetArg (args[n], XmNbackground, &bg);  n++;
        XtGetValues (XmSelectionBoxGetChild(dialogW,XmDIALOG_TEXT), args, n); 

        /* ... und als Vordergrundfarbe setzen */
        n = 0;
        XtSetArg (args[n], XmNbackground, bg);  n++;
        XtSetArg (args[n], XmNforeground, bg);  n++;
        XtSetArg (args[n], XmNcursorPositionVisible, False);  n++;
        XtSetValues (XmSelectionBoxGetChild(dialogW,XmDIALOG_TEXT), args, n); 
    }

    /* Text-Inhalt leeren */
    XmTextSetString (XmSelectionBoxGetChild(dialogW,XmDIALOG_TEXT), "");

    /* Exklusiv-Zugriff auf Keyboard setzen
     *  - alle Key-Events, die nicht fuer den Client sind,
     *    werden an das Root-Window geschickt
     */
    status = XGrabKeyboard (XtDisplay(dialogW),
                            RootWindowOfScreen(XtScreen(dialogW)),
                            True, GrabModeAsync, GrabModeAsync, CurrentTime);
    if (status != GrabSuccess) {
        fprintf (stderr, "can't grab keyboard\n");
        return;
    }

    /* Dialoganfrage ausgeben */
    XtManageChild (dialogW);
}

/* getpwEndeCB
 *  - Dialoganfrage zum Password-Einlesen beenden
 *  - eingelesenes Password auf Standard-Ausgabe ausgeben
 */
/*ARGSUSED*/
void getpwEndeCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (nicht verwendet) */
XtPointer aufrufDaten; /* Aufruf-Daten (nicht verwendet) */
{
    String password;

    /* Exklusiv-Zugriff auf Keyboard aufheben */
    XUngrabKeyboard (XtDisplay(widget), CurrentTime);
                                
    /* falls OK */
    if (((XmSelectionBoxCallbackStruct*)aufrufDaten)->reason == XmCR_OK) {
        /* eingegebenes Password auslesen (erzeugt Kopie) und ausgeben */
        password = XmTextGetString (XmSelectionBoxGetChild(widget,
                                                           XmDIALOG_TEXT));
        printf ("password: \"%s\"\n", password);

        /* ausgelesene Kopie freigeben */
        XtFree (password);
    }
}

/* exitCB
 *  - Programm beenden
 */
/*ARGSUSED*/
void exitCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (nicht verwendet) */
XtPointer aufrufDaten; /* Aufruf-Daten (nicht verwendet) */
{
    exit (0);
}
